# frozen_string_literal: true

require "dry-validation"

module OtbJobQueue
  # Parses a string of jobs into an hash
  class JobsParser
    JOBS_QUEUE_FORMAT = /(\w) => (\w|)/.freeze

    private_class_method :new

    class InputError < TypeError; end

    # Creates a new instance of the JobsParser with the valid jobs_input param
    # @param jobs_input [String] jobs string
    def initialize(jobs_input)
      @jobs_input = jobs_input
    end

    # JobsParser's Validation schema
    # Successful validation if input is an empty String or a String of valid format
    Schema = Dry::Validation.Schema do
      required(:jobs_input) { type?(String) & (empty? | format?(JOBS_QUEUE_FORMAT)) }
    end

    # Creates a new JobsParser instance and calls the .call method on it
    # @api public
    # @return [JobsParser.new(jobs_input).call, InputError] the jobs hash
    # @example
    #   OtbJobQueue::JobsParser.call('a => \nb => \nc => b')
    #     => { "a"=> [], "b"=> [], "c" => ["b"] }
    #   OtbJobQueue::JobsParser.call('Hello World!')
    #     => InputError ({ :jobs_input => ["must be empty or is in invalid format"] })
    def self.call(jobs_input)
      validation = self::Schema.call(jobs_input: jobs_input)
      if validation.success?
        new(jobs_input).call
      else
        raise InputError, validation.messages
      end
    end

    def call
      parse
    end

    private

    # Parses a string of jobs into an hash
    # @api private
    # @return [Hash] the jobs hash
    def parse
      @jobs_input.scan(JOBS_QUEUE_FORMAT).each_with_object({}) do |(job, dependency), memo|
        memo[job] = dependency.empty? ? [] : [dependency]
      end
    end
  end
end
