# frozen_string_literal: true

module OtbJobQueue
  # Sequences jobs order based on their dependencies
  class JobsSequencer
    class SelfDependencyError < StandardError; end
    class CircularDependencyError < StandardError; end

    # Creates a new instance of the JobsSequencer with a parser as a param
    # @param parser
    def initialize(parser)
      @parser = parser
    end

    # Parses an input of jobs into a string or raises Errors
    # @api public
    # @return [String] correct sequence of jobs
    # @example
    #   sequencer = OtbJobQueue::JobsSequencer.new(OtbJobQueue::JobsParser)
    #   sequencer.call("a => ") #=> "a"
    #   sequencer.call("a => b\nb => c\nc => ") #=> 'cba'
    def call(jobs_input)
      jobs_to_sequence = @parser.call(jobs_input)
      check_self_dependecies(jobs_to_sequence)
      jobs_to_sequence.tsort.join
    rescue TSort::Cyclic => error
      raise CircularDependencyError, "Jobs can't have circular dependencies: #{error.message}"
    end

    private

    # Verifies if the jobs_to_sequence has self dependencies
    # @api private
    # @param [Hash] jobs to sequence hash
    # @return [SelfDependencyError, void]
    def check_self_dependecies(jobs_to_sequence)
      jobs_to_sequence.each do |job, dependency|
        if job == dependency.first
          raise SelfDependencyError, "Jobs can't depend on themselves: #{job} => #{dependency.first}"
        end
      end
    end
  end
end
